;;I CHANGED THE WORD 'ERROR' FOR 'DEV' IN THIS SIMULATION BECAUSE THAT IS NOW A KEYWORD


extensions[array];;using extensions, though perhaps lists are more efficient?


globals[
data-array
data-list
short-array
inputs ;the lists that comprise the...
outputs ;;...training data
epoch-error
frame ;;the patches that conform the drawing space
count-examples ;;keep count of number of examples
example-1
example-2
example-3
example-4
example-target
]
links-own [weight]


breed [input-nodes input-node]
input-nodes-own [activation dev]
breed [output-nodes output-node]
output-nodes-own [activation dev]
breed [hidden-nodes hidden-node]
hidden-nodes-own [activation dev]


;;;
;;; SETUP PROCEDURES
;;;

to setup
  ;; (for this model to work with NetLogo's new plotting features,
  ;; __clear-all-and-reset-ticks should be replaced with clear-all at
  ;; the beginning of your setup procedure and reset-ticks at the end
  ;; of the procedure.)
  __clear-all-and-reset-ticks
  ask patches [ set pcolor 38 ]
  set count-examples 0
  set-default-shape input-nodes "circle"
  set-default-shape output-nodes "output-node"
  set-default-shape hidden-nodes "output-node"
  setup-nodes
  setup-links
  setup-frame
  setup-char-box
  set epoch-error 0
  make-labels
  end
  
to make-labels
  set example-target "example-1"
  ask patch 0 10 [set plabel-color 32 set plabel "input"]
  ask patch 5 10 [set plabel-color 32 set plabel "hidden"]
  ask patch 9 10 [set plabel-color 32 set plabel "output"]
  ask patch -5 8[set plabel-color 32 set plabel "Draw here"]
  ask patch -5 -3[set plabel "Example 1"]
  ask patch -1 -3[ set plabel "Example 2"]
  
  ask patch 3 -3[ set plabel "Example 3"]
  ask patch 7 -3[ set plabel "Example 4"]
end


to setup-nodes
;;sprout turtles in order from 0 to 14 in the same array as drawings
foreach sort patches with [pxcor >= -1 and pxcor < 2 and pycor = 6]
[ask ? [sprout 1 [set breed input-nodes]]]
foreach sort patches with [pxcor >= -1 and pxcor < 2 and pycor = 5]
[ask ? [sprout 1 [set breed input-nodes]]]
foreach sort patches with [pxcor >= -1 and pxcor < 2 and pycor = 4]
[ask ? [sprout 1 [set breed input-nodes]]]
foreach sort patches with [pxcor >= -1 and pxcor < 2 and pycor = 3]
[ask ? [sprout 1 [set breed input-nodes]]]
foreach sort patches with [pxcor >= -1 and pxcor < 2 and pycor = 2]
[ask ? [sprout 1 [set breed input-nodes]]]
;;give nodes a small original activation
ask input-nodes [ set activation random-float 0.1 ]

create-hidden-nodes 1 [ setxy 5 8 ]
create-hidden-nodes 1 [ setxy 5 6 ]
create-hidden-nodes 1 [ setxy 5 2 ]
create-hidden-nodes 1 [ setxy 5 0 ]
ask hidden-nodes [ set activation random-float 0.1
                    set size 1.5 ]
create-output-nodes 1 [ setxy 9 8 ]                
create-output-nodes 1 [ setxy 9 6 ]
create-output-nodes 1 [ setxy 9 2 ]
create-output-nodes 1 [ setxy 9 0 ]
ask output-nodes [ set activation random-float 0.1 set size 1.5]
recolor
end

;;Create the small frame where characters can be drawn
to setup-frame
ask patches[if pxcor = -8 or pxcor = -4
[if pycor <= 7  and pycor >= 2[set pcolor red]]
if pycor = 7 or pycor = 1[
if pxcor <= -4 and pxcor >= -8[set pcolor red]]
if (pxcor < -4 and pxcor > -8) and pycor <= 6 and pycor >= 2
[set pcolor white]

set frame patches with [pcolor = white]
]
end

to setup-char-box
ask patches[
if pycor = -3 or pycor = -9[
  if abs pxcor <= 8
   [set pcolor brown]]

if pycor < -3 and pycor > -9[
  if abs pxcor = 8 or abs pxcor = 4 or pxcor = 0
    [set pcolor brown]]]
    
set example-1 patches with 
  [pxcor <= -5 and pxcor >= -7 and pycor < -3 and pycor > -9]
set example-2 patches with 
  [pxcor <= -1 and pxcor >= -3 and pycor < -3 and pycor > -9]
set example-3 patches with 
  [pxcor <= 3 and pxcor >= 1 and pycor < -3 and pycor > -9]
set example-4 patches with 
  [pxcor <= 7 and pxcor >= 5 and pycor < -3 and pycor > -9]
end

to setup-links
  connect-all input-nodes hidden-nodes
  connect-all hidden-nodes output-nodes
end

;;get nodes to interconnect from input to hidden and hidden to output
to connect-all [nodes1 nodes2]
  ask nodes1 [
    create-links-to nodes2 [
      set weight random-float 0.2 - 0.1 ;;set weigths smaller than abs 0.1
    ]
  ]
end

;;keep nodes changing color according to activation level. light blue 0, yellow 1.
to recolor
  ask turtles with [breed != links] [
    set color item (step activation) [97 yellow]
  ]
  ask links [
    set thickness 0.05 * abs weight ;;reduce the thickening to half the original
    ifelse weight > 0
      [ set color red ]
      [ set color blue ]
  ]
end

;;;
;;; TRAINING PROCEDURES 
;;;

;;the training data consist on an input per each input node and an output per each output node for
 ;;the desired pairing 
to train-on-data
set epoch-error 0 ;set error to zero at start

set data-array array:from-list data-list ;;make array from data list

repeat examples-per-epoch [;;repeat the training as many times as defined by user

    let line 0
    repeat array:length data-array[ ;run through the whole array of data (here only four cases)

    set short-array array:item data-array line ;;this makes each list in the array a single list
;;brake each line into inputs (first 15 values) and outputs (values 15-19)
    set inputs sublist short-array 0 15  ;;last index in sublists is exclusive
    set outputs sublist short-array 15 19 
 
  
  ;;this loop sets the input nodes activation to the value in the data per each drawing
    (foreach [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14] [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14]
    [ask input-node ?1 [set activation  item ?2 inputs]])
  
    propagate ;;first pass on the net forward
    back-propagate outputs ;;send outputs to backpropagate error
    set line line + 1 ;go to next line in data array
    tick  
    ]
  plotxy ticks epoch-error ;;plot the error
  set epoch-error (epoch-error / examples-per-epoch) 
]
end


;;;
;;; PROPAGATION PROCEDURES (from original NetLogo Artificial Neural Net model) 
;;;

;; carry out one calculation from beginning to end
to propagate
  ask hidden-nodes [ set activation new-activation ]
  ask output-nodes [ set activation new-activation ]
  recolor
end

to-report new-activation  ;; node procedure applies sigmoid
  report sigmoid sum [[activation] of end1 * weight] of my-in-links
end

;;Back propagate receives the outpus
to back-propagate [values]

  let example-error 0

  (foreach [19 20 21 22] [0 1 2 3] ;;use the output values to calculate error for each data input
  [ask output-node ?1 [ set dev activation * (1 - activation) * (item ?2 outputs - activation) ]
    set example-error example-error + ( (item ?2 outputs - [activation] of output-node ?1) ^ 2 )
  ] )
  set epoch-error epoch-error + (example-error / count output-nodes)
  ask hidden-nodes [
    set dev activation *
              (1 - activation) *
              sum [weight * [dev] of end2] of my-out-links
  ]
  ask links [
    set weight weight + learning-rate * [dev] of end2 * [activation] of end1 ;;modify link weight
  ]
end

;; computes the sigmoid function given an input value and the weight on the link
to-report sigmoid [input]
  report 1 / (1 + e ^ (- input))
end

;; computes the step function given an input value and the weight on the link
to-report step [input]
  ifelse input > 0.5
    [ report 1 ]
    [ report 0 ]
end



;;INPUT PROCEDURES

;;Use mouse to change color of frame
to draw
  if mouse-down?    
    [
      let point patch mouse-xcor mouse-ycor
      if member? point frame [
      ask point [set pcolor black ]]
    ]
end

;;This procedure converts the frame patches into a set of inputs
;in order to use them in the net
to grab-pic
let pixels sort frame ;;assign each patch to an ordered agentset
set inputs [0]
foreach pixels [
ifelse [pcolor] of ?1 = black 
[set inputs lput 1 inputs ];if the color of pixel is black, input is 1
[set inputs lput 0 inputs]];;..else, it is 0
set inputs but-first inputs ;;remove the first zero
activate-sensors ;;get input-nodes to activate accordingly
assign-outputs
save-example count-examples
ask frame [set pcolor white]
make-data-list ;;send inputs and outputs to be entered into the same list
if count-examples = 4
[set data-list but-first data-list] ;;remove the first zero only once
end

;; assign outputs as 1000 0100 0010 0001
to assign-outputs
set outputs [0 0 0 0]
set outputs replace-item count-examples outputs 1
set count-examples count-examples + 1
end

to activate-sensors
(foreach inputs (sort input-nodes )
  [ask ?2 [set activation ?1]])
  recolor
end

to save-example [index]
show index
if index = 1
[(foreach inputs (sort example-1)
  [ask ?2 [ if ?1 = 1 [set pcolor 32]]])]
  if index = 2
[(foreach inputs (sort example-2)
  [ask ?2 [ if ?1 = 1 [set pcolor 32]]])]
  if index = 3
[(foreach inputs (sort example-3)
  [ask ?2 [ if ?1 = 1 [set pcolor 32]]])]
  if index = 4
[(foreach inputs (sort example-4)
  [ask ?2 [ if ?1 = 1 [set pcolor 32]]])]
end

;;put all data into a large list
to make-data-list
set data-list (sentence data-list (list sentence inputs outputs))
end


;;;
;;; TESTING PROCEDURES
;;;

;;To test the net, grab the values of the frame patches as before and
;;propagate the activation through the net
  
to test-net
let pixels sort frame
set inputs [0]
foreach pixels [
ifelse [pcolor] of ?1 = black
[set inputs lput 1 inputs ]
[set inputs lput 0 inputs]]
set inputs but-first inputs 
activate-sensors
propagate
end


;
; *** NetLogo 4.0.2 Model Copyright Notice ***
;
; Copyright 2006 by Uri Wilensky.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Rand, W. and Wilensky, U. (2006). NetLogo Artificial Neural Net model.
; http://ccl.northwestern.edu/netlogo/models/ArtificialNeuralNet.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2006 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/ArtificialNeuralNet
; for terms of use.
;
; *** End of NetLogo 4.0.2 Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
246
26
731
532
10
10
22.62
1
10
1
1
1
0
0
0
1
-10
10
-10
10
0
0
1
ticks
30.0

BUTTON
19
34
100
67
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
14
128
214
161
learning-rate
learning-rate
0.0
1.0
0.2
1.0E-4
1
NIL
HORIZONTAL

PLOT
13
209
213
359
Error vs. Epochs
Epochs
Error
0.0
10.0
0.0
0.5
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""

SLIDER
14
168
213
201
examples-per-epoch
examples-per-epoch
1.0
1000.0
1000
1.0
1
NIL
HORIZONTAL

BUTTON
21
82
126
115
NIL
train-on-data
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
843
193
967
226
NIL
test-net
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
839
66
902
99
NIL
draw
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
842
129
919
162
NIL
grab-pic
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
907
66
1002
99
clear
ask frame[set pcolor white]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

@#$#@#$#@
## WHAT IS IT?

This is a model of a character recognition backpropagation neural network. It is based on the NetLogo Artifical Neural Net model in the Models Library. This expanded version will accept four different 3 X 5 'pixel' drawings and then recognize them after the net is trained on the examples.  

## HOW IT WORKS

Initially the weights on the links of the networks are random. Inputs are entered into the input nodes by activating the 'draw' procedure and using the mouse to draw characters in the small frame to the left. After each character is drawn, pressing 'grab-pic' will use the drawing to activate the appropriate input nodes. The user can enter up to four different drawings, which will be then copied to the boxes below the net.   
The values of the inputs times the random weights are added up to create the activation for the next node in the network.  The next node then sends out an activation along its output link.  These link weights and activations are summed up by the final output node which reports a value.  This activation is passed through a sigmoid function, which means that values near 0 are assigned values close to 0, and vice versa for 1.  The values increase nonlinearly between 0 and 1 with a sharp transition at 0.5. Light blue nodes have an activation below 0.5, yellow nodes an activation above 0.5. 

To train the network the four inputs are presented to the network along with how the network should correctly classify the inputs. In this case, the outputs are simply the activation of the first, second, third and fourth output nodes for each drawing, in that order.  The network uses a back-propagation algorithm to pass error back from the output node and uses this error to update the weights along each link.

## HOW TO USE IT

To use it press SETUP to create the network and initialize the weights to small random numbers. This will also clear the small frame where the characters can be drawn.

Press DRAW to begin the drawing procedure and use the left button of the mouse to turn the white patches in the red frame to black. The drawings can be any pattern within the 3 X 5 rectangle. 

Press GRAB-PIC to let the network know you are done with the character or CLEAR-FRAME to draw something else without recording it as an example. After pressing GRAB-PIC you will notice that the input nodes 'copy' the shape of your character through their activations. At the same time, the boxes below the net will save the characters you have already entered. 

Once four characters have been added to the net, press TRAIN-ON-DATA and the net will present the four character inputs consecutively times the number of EXAMPLES-PER-EPOCH selected. 

As in the original model, the larger the size of the link, the greater the weight it has.  If the link is red then its a positive weight.  If the link is blue then its a negative weight.

After the net is done training, you can press DRAW again to draw an input to test. Pres the TEST-NET button when you are done and see which output node is activated. The net should be able to recognize the original four characters by activating the appropriate output node (1 through 4, in the order they were entered). 

LEARNING-RATE controls how much the neural network will learn from any one example.

## THINGS TO NOTICE

Now that the network has learned the examples, you can try to enter slightly different drawings. The net should be able to deal with 'noisy' inputs and yet recognize them as those that it was trained on. Adding or deleting 'pixels' from a test example will show you this is the case.

It is interesting to note that every time you run the net, even on the same test examples, the final result in terms of the weight and sign (positive or negative) of the links, more clearly visible in the links from the hidden nodes to the ouput nodes, is always slightly different. 

## THINGS TO TRY

Manipulate the LEARNING-RATE parameter.  Can you speed up or slow down the training?

The more similar the original drawings are to one another, the more difficult it is to train the net to recognize them correctly. Try entering a letter P and a letter F and see which 'pixels' seem to cause the net to recognize the test examples as one or the other.  


## EXTENDING THE MODEL

Back-propagation using gradient descent is considered somewhat unrealistic as a model of real neurons, because in the real neuronal system there is no way for the output node to pass its error back.  Can you implement another weight-update rule that is more valid?

## NETLOGO FEATURES

This model uses the link primitives.  It also makes heavy use of lists.

The extended character model also makes use of agentsets to define the frame and boxes. 

## RELATED MODELS

The two models in the Neural Net models library. Particularly the Artificial Neural Net of which this is an extension.

## CREDITS AND REFERENCES
This model is part of NeuroLab, a set of neuroscience-related simulations in NetLogo written by
 Luis F. Schettino
Psychology Department 
Lafayette College 
Easton, PA, USA. 
https://sites.lafayette.edu/schettil

From the original Artificial Neural Net Model:
The code for this model is inspired by the pseudo-code which can be found in Tom M. Mitchell's "Machine Learning" (1997).

Thanks to Craig Brozefsky for his work in improving this model.

To refer to this model in academic publications, please use:  Rand, W. and Wilensky, U. (2006).  NetLogo Artificial Neural Net model.  http://ccl.northwestern.edu/netlogo/models/ArtificialNeuralNet.  Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use:  Copyright 2006 Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/ArtificialNeuralNet for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bias-node
false
0
Circle -16777216 true false 0 0 300
Circle -7500403 true true 30 30 240
Polygon -16777216 true false 120 60 150 60 165 60 165 225 180 225 180 240 135 240 135 225 150 225 150 75 135 75 150 60

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

link
true
0
Line -7500403 true 150 0 150 300

link direction
true
0

output-node
false
1
Circle -7500403 true false 0 0 300
Circle -2674135 true true 30 30 240
Polygon -7500403 true false 195 75 90 75 150 150 90 225 195 225 195 210 195 195 180 210 120 210 165 150 120 90 180 90 195 105 195 75

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.4
@#$#@#$#@
setup repeat 100 [ train ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0

@#$#@#$#@
0
@#$#@#$#@
