 

globals
[
  ;;/////////////////////SETUP and AP GLOBALS/////////////
color-world      ;;so you can easily change the color of the world
tip ;the ycor of the lowest patch in the presynaptic membrane. Used to determine how far the AP should go
pre-synaptic   ;;global variable that contains all axon patches
pre-membrane ;;global variable that contains only the presynaptic 'membrane'
Gap ;;global variable that contains the patches in the synaptic gap
post-synaptic ;;all patches in postsynaptic membrane
post-membrane ;;the set of postsynaptic patches that touch the gap
f1 f2;;foci 1 and 2 for the ellipse used in drawing the world
release-region ;;part of the post-membrane where vesicles can dock
vesicle-strip ;a strip of the presyn membrane where vesicles are produced
;////////////////////////RUNTIME GLOBALS////////////
wavefront ;; global needed to draw the movement of the AP down the terminal
wave ; a set of patches that flashes to represent the AP in the Presynaptic structure
Particles ;; a global grouping the moving turtles
AP-ready ;boolean that flags when the AP can occur
releasable-NTs ;the actual number of NTs released by each vesicle (depends on CA entry)
ca-count ;; number of cs in the axon
;;////////////////////PLOT GLOBALS//////////////////
run-number ;to assign the bar its proper xaxis location
plot-it; boolean that tells the plot when to run
PSP-count ;;count the PSPs occurring within a time frame
penc] ;;pen colors for graph

patches-own[
PSP-ready  ;THIS IS a BOOLEAN THAT TELLS AN ENTERING ION IF THE PATCH HERE BELONGS TO POST-SYNAPTIC
psp ;;variable that determines the color they will flash
r1 r2 total ;;variables used to define location relative to ellipse
occupied];;used to determine whether an element is already at that location 


breed [NTs NT] ;;neurotransmitter breed
breed [vesicles vesicle]   ;;vessicles that carry recetors or neurotransmitters
breed [uptakes uptake] ;;receptors on presynaptic membrane
breed [channels channel] ;;receptors in postsynaptic membrane
breed [ca-channels ca-channel];;calcium receptors
breed [cs c] ;;calcium ions
breed [nas na] ;;sodium ions
breed [att-ions att-ion];;ions attracted by a receptor
breed [enzymes enzyme]
breed [autorecs autorec]
breed [ca-ins ca-in] ;ca-channel inhibitors

turtles-own 
[r-position] ;;used to place moving turtles within the gap

autorecs-own
[partner]

uptakes-own
[partner] 

channels-own
[partner
my-PSP ;all the patches that each ampa receptor owns
open
open-time]

NTs-own 
[partner ;;once an NT is bound to a receptor it partners with it
active] ;; whether the NT is in active state or not (boolean)

ca-channels-own
[partner;if there is a partner, do not open and change shape
 ca-blocked ;;boolean that will tell whether or not they are antagonized
open-time ;; time the receptors stay open
open] ;;whether the receptor is open at any given time (boolean)
 
att-ions-own [
fired ;;THIS IS A BOOLEAN THAT IS SET TO TRUE ONCE THE ION CROSSES INTO POST-SYNAPSE (ONLY NAS)
in-time ;;time attracted ions are allowed to remain within a structure... 
ID];;...before they are returned to the gap. ID = 1 cs ID = 2 nas

enzymes-own[
partner
block-time]

ca-ins-own
[partner
  active]

vesicles-own[activated target]
;;;;;;;;;;;;;;;;;;;;;;;;;;RUN-TIME PROCEDURES;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to Start
if ticks = 0[ ;;run these commands once at the start of every run
    set Particles turtles with [breed = nas or breed = enzymes or breed = cs]
    set PSP-count 0]
 if count NTs with [active = true and color = green] = 0 and PSP-count != 0 [Stop-procedures stop]
 
 if ticks > 1500 and count NTs with [color = turquoise] > 0 [Stop-procedures repeat 7 [ask min-one-of vesicles [ who] [die]] stop]
 
 if ticks > 2000 [Stop-procedures stop]
    
    go
   
end
 
to go
  
tick

ask Particles [move] ;;asks ions and enzymes to move
                           
;;The following two statements are needed because neither NTs nor att-ions are created at time zero                           
ask NTs [if partner = nobody [move]] 
ask att-ions[move]
ask att-ions with [id = 2] [check-fire] ;;ONLY NAS ATT-IONS RUN PROCEDURE to show PSP


;control the uptake and autoreceptors
ask turtles [if (breed = uptakes or breed = autorecs)
               [bind-active-nts
                if partner != nobody [diminish]]]

ask channels [attract control-receptors  recolor]

release-nts? ;;NTs are only released if enough CA ions enter the terminal

remove-NTs;;inactive NTs are killed of one-by-one every 100 iterations

ask enzymes with [shape = "enzyme"] [inactivate-nts ] 

advance-wave

end


to Stop-procedures
  set plot-it true 
Clear-Particles
reset-ticks
set plot-it false
set penc 1


end

to Clear-Particles

ask NTs[die]
ask channels [set size 20 
              set color red 
              set partner nobody 
              set shape "receptor" 
              set open false 
              set open-time 0]


ask ca-channels [set ca-blocked false]
ask att-ions [set in-time 0]

end

;;////////////////////////////////////////////MOTION PROCEDURES///////////////////////////////
;;Procedure move controls the motion of all turtles that move. 


to move

if breed = NTs and partner != nobody [stop] ;;if bound-NT, quit the procedure
bounce ;;look ahead one patch and see whether you need to turn your heading

fd 0.3
set heading heading + (random 4 - 1.5)

if breed = att-ions [Pop-out]
end

to bounce
 let new-patch patch-ahead 1
 if new-patch = nobody [random-position stop] 
 ;;the next two statements prevent particles from moving too close to limits of world
 if [pycor] of new-patch > max-pycor - 3 [set heading (heading - 175 - random 30)]         
 if [pxcor] of new-patch > max-pxcor - 3 or [pxcor] of new-patch < min-pxcor + 3 [set heading (heading - 150 - random 60)]
 

 if breed = att-ions [stop]
 ;we use larger or equal to color world because the PSP flash tends to allow particles in
 if [pcolor] of new-patch >= color-world  [set heading (heading - 150 - random 60)]; this is the regular bounce for all particles
 if [pcolor] of patch-here = color-world [random-position ] ;;this tells them to move out if they sneak in somehow

 end

;;let ions come into structure and live for 50 iterations. Then place back in extracellular space
to Pop-out
if ticks > in-time + 50 [
     ifelse ID = 1 
     [set breed cs ifelse who mod 2 = 0 [set r-position 1] [set r-position 0] random-position]
     [set breed nas set r-position 2 random-position]]
end

;;;;;;/////////////////////////////////////NT RELEASE PROCEDURES////////////////////////////////

to release-nts?
 let max-ca-count 0
set ca-count count att-ions with [ID = 1];count the calcium ions
let temp ca-count  ;and grab the highest number before it goes down
if max-ca-count < temp ;bubble up the max count of CAs
[set max-ca-count temp]

;modulate the amount of NT molecules depending on the influx of CA
ifelse max-ca-count > 10
[set releasable-NTs Number-of-NTs]
[ifelse max-ca-count > 8
  [set releasable-NTs Number-of-NTs * 0.75]
  [ifelse max-ca-count > 5
    [set releasable-NTs Number-of-NTs * 0.4]
    [set releasable-NTs 1]]]

end

;RELEASE THE NTs by the vesicles

to release-NTs
hatch releasable-NTs [set breed NTs
         set heading 165 + random-float 30 
         set color green
         set size 15
         set shape "neurotransmitter"
         set partner nobody
         set active true
        
         fd 5]
end

;;Inject produces a local variable at the lowest tip of the presynaptic structure. Then it runs down the presynaptic
;;membrane one row at a time asking them to turn their color lighter. It is called by the AP button
to inject 
if wavefront > tip  ;; if the wave has not yet reached the bottom of ellipse
;;move wavefront down one line of patches 
[set AP-ready true
 ] ;let the system know a run has begun

end

;;called by Inject to color the presynaptic membrane
to advance-wave
  ifelse  AP-ready = true
  [
    ifelse wavefront >= tip
      [
       set wavefront (wavefront - 1)
        (foreach [0 1 2 3 4] [4 3.5 3 2 1] ;;ask four rows below wavefront to take lighter colors
          [ask wave with [pycor = wavefront - ?1] [set pcolor yellow + ?2]])

        ask wave with [pycor > wavefront] [set pcolor color-world]
        
        if  wavefront < 40[
                       ask vesicles [activate-vesicles set-vesicles-in-motion]]] ;if calcium came in, release NTs]
      [set AP-ready false set wavefront max-pycor ask vesicle-strip [set occupied false]  make-vesicles]]

 [stop ] 
end


to activate-vesicles
 set activated true  
  set color yellow 
end

;all vesicles begin to move until they reach the membrane
to set-vesicles-in-motion

   ifelse [total] of patch-here >= 257
    [fuse]
    [fd 1]
  
end


to fuse
  set shape "open-ves"
  release-NTs
  die
end

;;///////////////////////////////////////CHANNEL PROCEDURES/////////////////////////////////////////


;;This procedure attracts NTs, agonists and antagonists into the channels
to attract
ifelse partner != nobody ;;if not taken
    [if ticks > open-time + 40 [  ;;if already have a partner for 40 iterations, open the pore and stop attracting
     
       open-up stop ]]
                        
    [
     if any? NTs in-cone 8 60 with [active = true and color = green] ;; grab a partner that is an actual NT
         [set partner one-of NTs in-cone 8 60 
        
          set shape "bound-receptor";;set shape to bound receptors
          
            
          ask partner [bind] ;;ask NT to hide
          set open-time ticks] ];;get your counter running
end

to bind
set partner (myself) ;;set me as your partner
set hidden? true
end


;;This procedure controls the opening and closing of CA and Na (ampa)receptors by manipulating a
;;boolean 'open' variable

to control-receptors
ask ca-channels [
  
                  check-for-blockage ;;change name to inhibited?
                  if ca-blocked = false [
                                         if pycor = wavefront and open = false [set open-time ticks open-up] ;;if the wave reaches your pycor open
                                         if open = true [ ;;if receptors are already open...
                                                         ifelse ticks > open-time + 70 ;;and time to close is up, close.
                                                               [close]
                                                               [ions-in] ] ]] ;;otherwise let in ions
ask channels [  
                ifelse open = false 
                    [stop]
                    [ifelse ticks > open-time + 400 ;;if already open and time to close is up, close.
                           [close]
                           [ions-in] ] ]  ;;otherwise let in ions
end


;; This procedure asks ions in a cone in front of the receptor to set their heading towards
;;the receptor and set their breed to attracted ions. This releases the ions from the constraints
;;of their former breed 
to ions-in
let my-xcor [pxcor] of patch-here 
let my-ycor [pycor] of patch-here
ifelse breed = ca-channels
    [ask cs in-cone 10 80 [
                       set breed att-ions set ID 1 ;;cs have ID = 1
                       set in-time ticks ;;set timer on to figure out when to get out
                       facexy my-xcor my-ycor ]] ;;look at receptor and increase speed 
    [ask nas in-cone 10 80 [
                       set breed att-ions set ID 2 ;;nas have ID = 2
                       set fired false            
                       set in-time ticks ;;set timer on to figure out when to get out
                       facexy my-xcor my-ycor ]] ;;look at receptor and increase speed
end



;;These procedures make Ca and ampa receptors open and close to let ions in
to open-up
ifelse open = true 
      [stop]
      [set shape "open-receptor" set size 30 set open true ] 
      if breed = channels [set PSP-count PSP-count + 1] ;;add one to the count of PSPs but only for Na channels
end

to close
ifelse open = false
   [stop]
   [set open false  set size 20 set open-time 0 
   if breed = ca-channels [set shape "ca-receptor"]
   if breed = channels [unbind-NT]]
end

;;Make sure all NT variables are set to inactive form and receptor variables are completely reset
to unbind-NT
ask partner [set color magenta
             set hidden? false
             set active false
             set heading 30 - random 15 ;;make NT float upwards
             fd 3
             set partner nobody
             ]
set color red
set partner nobody
set shape "receptor"  
set size 20 
set open false
set open-time 0          
end



to check-fire
if fired = true 

  [stop] ;;if you have already fired, stop

  ask patch-here [if PSP-ready = true[ ;;else if the patch where you find yourself is postsynaptic
    ask channels in-radius 8 [fire-PSP]]];;ask the ampa receptor close-by to fire
    
 if [pcolor] of patch-here != black ;;if you are within the membrane
  [set fired true]

end

;;This procedure draws the PSP flash by increasing variable psp on the
;;postsynaptic patches surrounding the ampa receptore
to fire-PSP
   ask my-PSP in-radius 8 [set psp psp + .75]  ;AMPAS WILL LAY DOWN PSP
   ask my-PSP in-radius 6 [set psp psp + 1 ]
   ask my-PSP in-radius 4 [set psp psp + 2 ]
   ask my-PSP in-radius 2 [set psp psp + 4 ]
   ask my-PSP in-radius 1 [set psp psp + 5 ];]
   ask my-PSP [set PSP-ready true]
end

;;"Evaporate" the variable psp to recolor the postsynaptic structure after a flash
to recolor ;;WHEN THERE ARE IONS ON THE AMPA'S PATCHES WE WILL RUN THIS
if any? my-PSP with [psp > 0]
   [ask my-PSP  [set psp (psp * (100 - 5) / 100) ;;;SERVES TO EVAPORATE THE PSP AT EVERY ITERATION
                set pcolor scale-color 98 psp -4 4] ] ;WE SET THE COLOR IN ACCORDANCE TO HOW MUCH PSP IS HERE. 
                
end

;;/////////////////////////////////ANTAGONISM AND ENZYME PROCEDURES///////////////////////////////////

to check-for-blockage

ifelse partner = nobody
  [
  let p patch-here 
  let degree heading 
  if any? nts in-cone 10 60 with [shape = "ca-antagonist" and color = turquoise ] 
   [
    set partner one-of nts in-cone 10 60 with [shape = "ca-antagonist" and color = turquoise ]  
                              ask partner[
                                set partner myself
                                set heading degree
                                setxy [pxcor] of p [pycor] of p
                                fd 1 ] 
                                set ca-blocked true ]
     ]
  
   [stop]
                                                                       
end



 ;;ENZYMES SELECT AN NT MOLECULE AND ASK IT TO MOVE ALONG FOR A SHORT TIME
to inactivate-nts
  ifelse partner != nobody ;if enzyme already has a partner
  
     [
       ifelse ticks > block-time + 500 ;if block time has passed
            [ ask partner [set partner nobody]
              set partner nobody set block-time 0 
             ];;release partner, reset vars
      
             ;otherwise maintain partner along
            [let lookinat [heading] of self ;save heading of enzyme
             let x-cor [xcor] of self ;save my x..
             let y-cor [ycor] of self;..and y cors
             ask partner[
                     set heading lookinat ;;and become inactive
                     setxy x-cor y-cor]];and keep on
     ]
  
     [if any? NTs with [color = green and partner = nobody] in-radius 4 ;if there are any NTs in radius 4 
       [
        let lookinat [heading] of self ;save heading of enzyme
        let x-cor [xcor] of self ;save my x..
        let y-cor [ycor] of self;..and y cors
       
        set partner one-of NTs with [color = green] in-radius 5 ;set as partner one of the NTs around
        set block-time ticks ;save the iteration as time zero
        ask partner[
                     set partner myself
                     set heading lookinat ;;and become inactive
                     setxy x-cor y-cor 
                     set color magenta 
                     set active false]]]
end

 ;;Remove on of the inactive NTs every 30 iterations. Check if they have a partner to prevent 
 ;;the channels from asking a dead partner to inactivate
to remove-NTs
if ticks mod 35 = 0 
   [if any? NTs with [active = false and partner = nobody]
       [ask one-of NTs with [active = false and partner = nobody]   [die]  ]]
end

;;This is the procedure for autoreceptors and re-uptake receptors

to bind-active-nts
  
   let lookinat [heading] of self
   let xc [pxcor] of patch-ahead 1;slightly ahead to look as though it is bound
   let yc [pycor] of patch-ahead 1
  
  ifelse breed = uptakes
   ;uptakes bind NTs and SRIs
    [ifelse partner = nobody 
       [
        if any? nts with [color = green or color = yellow] in-cone 10 45 ;;if NTs around...
                  [set partner one-of nts with [color = green or color = yellow] in-cone 10 45
                    ask partner[
                                setxy xc yc 
                                set heading lookinat 
                                set partner myself ]]]
      
                         [stop]
    ]                                        

     [ifelse partner = nobody 
       [
        if any? nts with [color = green ] in-cone 10 45 ;;if NTs around...
                  [set partner one-of nts with [color = green ] in-cone 10 45
                    ask partner[
                                setxy xc yc 
                                set heading lookinat 
                                set partner myself ]]]
      
                         [stop]
      ]  


end
   
to diminish
  
  ifelse [color] of partner = green
  
    [if size <= 9 [ask partner [die] set partner nobody set size 14  stop]
  set size (size - .1)
  ask partner [set size (size - .1)]]
    [stop]
end

;;;;;;;;;;;;;;;;;SETUP PROCEDURES;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to setup
ca
set color-world sky ;;to change the color of the world insert your color

set-globals
make-world 
add-membrane-structures
reset-ticks
end

;;Make-world controls the drawing of the pre and postsynaptic structures by calling
;;to ellipse and using its outputs to determine patch location 
to make-world
 

ask patches[set pcolor color-world set psp 0 set PSP-ready false];;SETTING PSP VARIABLES
ask patches [ellipse] ;;Define an ellipse to serve as scaffolding for the synaptic structures     
ask patches with [total < 310] [set pcolor black] ;;define the synaptic gap
set post-synaptic patches with [pycor <= 0 and pcolor = color-world] ;;define postsynaptic structure
ask post-synaptic [set PSP-ready true] ;;ALL PATCHES ARE FIRST INITIALIZEZ WITH PSP-ready TO FALSE.?????
set post-membrane post-synaptic with [any? neighbors with [pcolor = black]] ;;define postsynaptic membrane
set pre-synaptic patches with [total < 260] ;;define presynaptic structure
set wave patches with [total < 260 and total > 250]
set pre-membrane patches with [total < 260 and total > 256] ;;define presynaptic membrane
set release-region patches with [total = 257 and pycor < 15]
ask pre-synaptic [set pcolor color-world ]
set Gap patches with [total < 290 and total > 260] ;;set synaptic gap
set tip ([pycor] of min-one-of pre-membrane [pycor]);;set local variable tip to the tip of presynaptic structure
set wavefront max-pycor ;;move this to a set-variables procedure 
set vesicle-strip patches with [total > 233 and total < 240]
make-particles 
    
end

to set-globals
set plot-it false

set AP-ready false

set penc 1

end

;;this procedure produces an ellipse centered on foci f1 and f2. Patches have a variable that defines their equal distance
;;from the foci (variable total). Depending on that distance, the pre and postsynaptic structures are defined
to ellipse
set f1 -100
set f2 100
set r1 round (distancexy f1 75)
set r2 round (distancexy f2 75)
set total r1 + r2
end

;;patches have a variable 'occupied' that lets them know whether they can sprout or not a vessicle or receptor
to add-membrane-structures
ask patches [set occupied false]
make-vesicles
while [count uptakes < 13][make-uptake]
while [count autorecs < 2][make-autorecs]
while [count channels < 12][make-channel]
while [count ca-channels < 2][make-ca-channels 1]
while [count ca-channels < 4][make-ca-channels 2]
end

to make-particles
make-cs
make-enzymes
make-nas
update-particles
set-default-shape att-ions "circle"
end

to update-particles
set Particles turtles with [breed = cs or breed = nas or breed = enzymes]
end

to make-nas
set-default-shape nas "circle"
create-nas 150
[set color orange ;color-world - 20 ;;random color based on world color
         ;;give it a decent starting speed
  set r-position 2    ;;the turtles use this position to determine where they will be born
   random-position     ;;carry this variable to the random position procedure
   set size 1.4]       
end

to make-cs
set-default-shape cs "circle"  
create-cs 120 
[set color yellow    ;;another random color

 set size 1.25
 ;;this is designed to have half of the cs with a position right
 ifelse who mod 2 = 0 [set r-position 1] [set r-position 0 ] ;;and half of position to the left
 random-position]      ;;carry this variable over to random position procedure
   
 end
 
 to make-enzymes
 create-enzymes 6 
 [set shape "enzyme"
 set color 135.9

 set size 15
 set partner nobody
 set r-position 2
 random-position
 ]
 end

to random-position
if r-position = 1 [
                    let location one-of Gap with [pxcor < -130 and pycor < 80] ;;Keep calcium ions at top of world
                    setxy ([pxcor] of location) ([pycor] of location)] 
if r-position = 0 [
                    let location one-of Gap with [pxcor > 130 and pycor  < 80]
                    setxy ([pxcor] of location) ([pycor] of location)]
if r-position = 2 [
                    let location one-of Gap with [abs pxcor < 100 and pycor < 0]
                    setxy ([pxcor] of location) ([pycor] of location)] 
if r-position = 3 [
                    let location one-of Gap with [abs pxcor > 65 and abs pxcor < 110]
                    setxy ([pxcor] of location) ([pycor] of location)] 


set heading random-float 360     ;;give them a random heading
end
 
 
 ;given that the PSP count depends strongly on the location of NT release, the location of the vesicles has to be
 ;defined better than that of other structures
to make-vesicles
  ;we ask patches in the vesicle-strip with pxcor between -75 and 75 to sprout the vesicles
  
  let locations [-75 -50 -25 0 25 50 75]
  
  foreach locations [ask one-of vesicle-strip with [pxcor = ?]
    
                            [sprout 1 [set breed vesicles 
                                       set color 118 ]]]
                                       
                                       ask vesicles[
                                       set size 8 
                                       set shape "NT-Ves"
                                       set activated false
                                       
                                       set target min-one-of release-region [distance myself] ;ask each vesicle to select a target patch in release region
                                       face target] ;;then face it
                                       

end
to make-uptake
ask one-of pre-membrane with [abs pxcor < 95]
                 [ if occupied = false ;;about right location on ellipse
                            [sprout 1 [set breed uptakes 
                                       set color brown  
                                       set size 14 
                                       set shape "auto-receptor"
                                       set partner nobody
                                       facexy 0 max-pycor 
                                       rt 180 ] ;;
                                       ask pre-membrane in-radius 10 [set occupied true]]]

end
to make-autorecs
ask one-of patches with [abs pxcor = 115 and total = 255] [if (occupied = false)
                            [sprout 1 [set color pink
                                       set size 20
                                       set breed autorecs
                                       set shape "auto-receptor"
                                      
                                       set partner nobody
                                       facexy 0 100
                                       rt 180 fd 2]
                                       ask patches in-radius 6 with [total = 255][set occupied true]]]
end  
to make-channel
ask one-of post-membrane with [abs pxcor < color-world] [if (occupied = false) ;;about right location on ellipse
                            [sprout 1 [set color red  
                                       set size 20 
                                       set breed channels 
                                       set partner nobody
                                       set shape "receptor"
                                       set open false
                                       set open-time 0
                                       set my-PSP post-synaptic in-radius 8 ;;THESE PATCHES ARE THE ONES THAT ARE ASKED TO FIRE OR SET PSP
                                       ask my-PSP [set PSP-ready true]
                                       facexy 0 75 ] ;;have receptors face the center of the structure to orient them 
                                       ask post-membrane in-radius 10 [set occupied true]]]
end
                                                       
to make-ca-channels [pos]
if pos = 1
[ask one-of patches with [pxcor > 125 and pycor < 85 and total = 258] 
                [if (occupied = false) ;;about right location on ellipse
                            [sprout 1 [set breed ca-channels
                                       set color red 
                                       set partner nobody
                                       set open-time 0 set open false
                                       set size 20 
                                       set ca-blocked false
                                       set shape "ca-receptor"
                                       facexy 0 75 set heading (heading + 180)] ;;have receptors face the center of the structure  
                                       ask patches in-radius 7 with [total = 258] [set occupied true]]]]
 if pos = 2                                      
[ask one-of patches with [pxcor < -125 and pycor < 85 and total = 258] 
                [if (occupied = false) ;;about right location on ellipse
                            [sprout 1 [
                                       set breed ca-channels
                                       set color red
                                       set partner nobody 
                                       set open-time 0 set open false
                                       set size 20 
                                       set ca-blocked false
                                       set shape "ca-receptor"
                                       facexy 0 75 set heading (heading + 180)] ;;have receptors face the center of the structure  
                                       ask patches in-radius 7 with [total = 258] [set occupied true]]]]
                                       
                                       
                                       
end 

to make-ca-channel-antagonist
 create-NTs 10
 [set shape "ca-antagonist"
 set color turquoise
 set partner nobody
 
 set size 15
 
 ifelse who mod 2 = 0 [set r-position 0]
                       [set r-position 1]
  random-position]
  update-particles
  set penc 2
 end

to make-SRIs
  ;;make a few more NTs with a different color that bind to reuptake pumps
create-NTs Number-of-SRIs[
        set color yellow
         set size 12
         set shape "uptake-inhibitor"
         set partner nobody
         set active true
         
         set r-position 2
 random-position
         
]
set penc 3
end
@#$#@#$#@
GRAPHICS-WINDOW
410
24
1323
598
150
90
3.0
1
16
1
1
1
0
0
0
1
-150
150
-90
90
0
0
1
ticks
30.0

BUTTON
47
22
114
55
NIL
Setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
49
97
173
141
NIL
Start
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
230
95
324
150
AP
inject
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
161
26
333
59
Number-of-NTs
Number-of-NTs
0
5
5
1
1
NIL
HORIZONTAL

BUTTON
49
389
192
422
Ca Channel blockers
make-ca-channel-antagonist
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
221
392
361
425
Reuptake Inhibitors
make-SRIs
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
211
449
383
482
Number-of-SRIs
Number-of-SRIs
0
20
18
1
1
NIL
HORIZONTAL

MONITOR
48
502
120
547
NIL
PSP-count
17
1
11

MONITOR
147
502
219
547
Active NTs
count NTs with [active = true and color = green]
17
1
11

PLOT
44
202
367
352
PSP counts
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"let pen-colors [ 65 135 0]\nset-plot-pen-color item (penc - 1) pen-colors" ""
PENS
"default" 1.0 1 -16777216 true "" "if plot-it = true [plot PSP-count]"

@#$#@#$#@
## WHAT IS IT?

This is a simulation of the classic neuronal synapse. The top blue structure represents the axonal terminal and the bottom blue structure the postsynaptic membrane. As the action potential reaches the axonal terminal, calcium channels (red) on the sides of the terminal open and allow the influx of calcium ions that facilitate the docking of neurotransmitter (NT) vesicles (purple circles) onto the presynaptic membrane with the consequent release of NT molecules (green triangles) into the synaptic gap (black space). 
As NT molecules traverse the gap, they bind to receptors that open to allow the flow of ions into the postsynaptic membrane, which causes local postsynaptic potentials (PSPs). 

## HOW TO USE IT

Use the speed slider to slow down the simulation a bit. The press the SETUP button to create the structures and populate the gap with calcium (yellow) and other ions (orange). Pressing the Start button will start the simulation. The AP (action potential) button will result in a depolarization of the axon terminal (traveling wave) that opens the calcium channels and activates the NT vesicles. 
As NT molecules are released into the gap and receptors open their channels, the PSP counter will add one for every time a channel opens. 

## THINGS TO NOTICE

A number of mechanisms exist to ensure that the communication between neurons is brief. Among these, there are reuptake transporters on the presynaptic membrane (brown) that bind the excess NT molecules and reabsorb them into the terminal. Also, enzymatic degradation of the NT molecules occurs when enzyme molecules (pink) in the gap bind to NTs. 

## THINGS TO TRY

After you have run the simulation with only NTs in the gap, add calcium blockers or reuptake antagonists to see their effects. Calcium channel blockers will prevent the influx of calcium ions reducing the possibility of NT release. In contrast, reuptake antagonists will block reuptake transporters, allowing NTs to remain in the gap for longer periods. 
As in natural synapses, the number of PSPs produced by a given action potential reaching the presynaptic membrane is somewhat variable. Therefore, it is a good idea to run a few trials to see the 'average' results of your manipulations.

## NETLOGO FEATURES

A lot of behind the scenes work goes into drawing the structures and adding the diverse types of agents. However, this is done at Setup, leaving only a relatively small set of methods to control the model during run-time.  

## RELATED MODELS

The Antagonism model of NeuroLab explores similar mechanisms, including a comparison between selective and non-selective antagonists. 

## CREDITS AND REFERENCES
This model is part of NeuroLab, a set of neuroscience-related simulations in NetLogo written by
Luis F. Schettino
Psychology Department 
Lafayette College 
Easton, PA, USA. 
https://sites.lafayette.edu/schettil
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

auto-receptor
true
11
Polygon -8630108 true true 113 115 98 115 98 147 203 147 203 114 190 114 199 123 106 123 113 115

bound-receptor
true
0
Polygon -7500403 true true 105 150 105 90 150 120 195 90 195 150 105 150
Polygon -13840069 true false 105 90 195 90 150 120 105 90

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

ca-antagonist
true
13
Polygon -7500403 true false 194 120 175 103
Polygon -7500403 true false 193 118 195 119
Polygon -2064490 true true 101 117 90 129
Polygon -2064490 true true 195 121 209 135 210 108 205 100 197 95 188 88 179 84 171 83 165 82 157 82 148 82 148 91 155 91 161 96 169 102 174 105 177 109
Polygon -2064490 true true 105 121 91 135 90 108 95 100 103 95 112 88 121 84 129 83 135 82 143 82 152 82 152 91 145 91 139 96 131 102 126 105 123 109

ca-receptor
true
1
Rectangle -2674135 true true 106 120 196 150
Circle -2674135 true true 120 90 60
Polygon -2674135 true true 105 120 123 104 125 102 131 98 117 126 105 120
Polygon -7500403 true false 194 120 175 103
Polygon -2674135 true true 194 118 170 98 174 127 194 119
Polygon -7500403 true false 193 118 195 119
Polygon -2674135 true true 192 117 193 120 185 120 191 117
Polygon -2674135 true true 190 117 192 126 192 131 161 144 146 125 159 109 138 108 130 114

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

enzyme
true
0
Circle -7500403 true true 105 150 30
Circle -7500403 true true 165 150 30
Polygon -7500403 true true 188 177 213 147 213 135 206 119 192 105 174 96 154 89 150 89 150 103 198 151 174 151 189 176
Polygon -7500403 true true 112 177 87 147 87 135 94 119 108 105 126 96 146 89 150 89 150 103 102 151 126 151 111 176

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

neurotransmitter
true
0
Polygon -7500403 true true 105 150 150 105 195 150 105 150

nt-ves
false
0
Circle -7500403 true true 0 0 300
Circle -8630108 true false 30 30 240

open-receptor
true
0
Polygon -2674135 true false 158 184 158 136 158 132 160 127 163 124 168 124 171 124 175 126 176 132 176 138 176 146 175 154 172 158 169 166 165 174 162 180 158 184
Polygon -7500403 true true 157 131 177 170
Polygon -2674135 true false 143 184 143 136 143 132 141 127 138 124 133 124 130 124 126 126 125 132 125 138 125 146 126 154 129 158 132 166 136 174 139 180 143 184

open-ves
true
0
Circle -7500403 true true 0 0 300
Circle -8630108 true false 30 30 240
Polygon -16777216 true false 45 30 60 75 105 105 150 135 195 120 240 90 270 60 240 15 180 0 135 -15 75 15 45 45

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

receptor
true
0
Polygon -7500403 true true 105 150 105 90 150 120 195 90 195 150 105 150

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

uptake-inhibitor
true
0
Polygon -7500403 true true 105 150 90 120 90 90 210 90 210 120 195 150

vessicle
true
0
Circle -7500403 true true 89 89 122
Circle -16777216 true false 100 100 100

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
